# Calendar.social
# Copyright (C) 2018 Gergely Polonkai
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from datetime import datetime
from functools import wraps
import os

from flask import Flask, current_app, redirect, render_template, url_for
from flask_babel import Babel, get_locale as babel_get_locale
from flask_security import SQLAlchemyUserDatastore, current_user, login_required


def get_locale():
    """Locale selector

    Selects the best locale based on values sent by the browser.
    """

    from flask import request

    supported_languages = ['en', 'hu']

    if 'l' in request.args and request.args['l'].lower() in supported_languages:
        return request.args['l'].lower()

    return request.accept_languages.best_match(supported_languages)


def template_vars():
    return {
        'lang': babel_get_locale().language,
    }


def route(*args, **kwargs):
    def decorator(func):
        setattr(func, 'routing', (args, kwargs))

        return func

    return decorator


class CalendarSocialApp(Flask):
    def __init__(self, name, config=None):
        from .models import db, User, Role
        from .security import security

        Flask.__init__(self, name)

        config_name = os.environ.get('ENV', config or 'dev')
        self.config.from_pyfile(f'config_{config_name}.py', True)
        # Make sure we look up users both by their usernames and email addresses
        self.config['SECURITY_USER_IDENTITY_ATTRIBUTES'] = ('username', 'email')
        db.init_app(self)
        babel = Babel(app=self)
        babel.localeselector(get_locale)

        user_store = SQLAlchemyUserDatastore(db, User, Role)
        security.init_app(self, datastore=user_store)

        self.context_processor(template_vars)

        for attr_name in self.__dir__():
            attr = getattr(self, attr_name)

            if not callable(attr):
                continue

            args, kwargs = getattr(attr, 'routing', (None, None))

            if args is None:
                continue

            self.route(*args, **kwargs)(attr)

    @route('/')
    def hello(self):
        from .calendar_system.gregorian import GregorianCalendar

        if not current_user.is_authenticated:
            return render_template('welcome.html')

        calendar = GregorianCalendar(datetime.utcnow().timestamp())

        return render_template('index.html', calendar=calendar)

    @route('/register', methods=['POST', 'GET'])
    def register(self):
        if not current_app.config['REGISTRATION_ENABLED']:
            return render_template('registration-disabled.html')

        from .forms import RegistrationForm
        from .models import db, User

        form = RegistrationForm()

        if form.validate_on_submit():
            # TODO: This might become False later, if we want registrations to be confirmed via E-mail
            user = User(active=True)

            form.populate_obj(user)
            db.session.add(user)
            db.session.commit()

            return redirect(url_for('hello'))

        return render_template('registration.html', form=form)

    @route('/new-event', methods=['GET', 'POST'])
    @login_required
    def new_event(self):
        from .forms import EventForm
        from .models import db, Event

        form = EventForm()

        if form.validate_on_submit():
            event = Event(user=current_user)
            form.populate_obj(event)

            db.session.add(event)
            db.session.commit()

            return redirect(url_for('hello'))

        return render_template('event-edit.html', form=form)


app = CalendarSocialApp(__name__)
